@@ -1,14 +1,50 @@
 ## location of Ensembl specific functions
 
+.getArchiveList <- function(https = TRUE, httr_config = list()) {
+  
+  url_worked <- FALSE
+  mirrors <- c("www", "asia", "uswest", "useast")
+  protocol <- ifelse(https, "https://", "http://" )
+  
+  while(!url_worked) {
+    
+    if(length(mirrors) == 0) {
+      stop("Unable to contact any Ensembl mirror")
+    }
+
+    mirror_option <- mirrors[1]
+    mirrors <- mirrors[-1]
+
+    url <- paste0(protocol, mirror_option, ".ensembl.org/info/website/archives/index.html?redirect=no")
+  
+    html <- GET(url, config = httr_config)
+  
+    ## this is TRUE if there's an HTTP error or we get the Ensembl error page
+    if(identical(status_code(html), 200L) && 
+       !grepl("The Ensembl web service you requested is temporarily unavailable", content(html))) {
+      return( content(html) )
+    }
+      
+  } 
+
+}
+
 ## scrapes the ensembl website for the list of current archives and returns
 ## a data frame containing the versions and their URL
-listEnsemblArchives <- function(https = TRUE) {
+listEnsemblArchives <- function(https) {
+    
+  if(!missing(https)) {
+    warning("Ensembl will soon enforce the use of https.\n",
+    "As such the 'https' argument will be deprecated in the next release.")
+  }
+  https <- TRUE
+    
+  .listEnsemblArchives(https = https, httr_config = list())
+}
+
+.listEnsemblArchives <- function(https = TRUE, httr_config) {
   
-  url <- ifelse(https,
-                "https://www.ensembl.org/info/website/archives/index.html",
-                "http://www.ensembl.org/info/website/archives/index.html")
-  
-  html <- xml2::read_html(GET(url))
+  html <- .getArchiveList(https, httr_config)
   html <- htmlParse( html )
   
   archive_box <- getNodeSet(html, path = "//div[@class='plain-box float-right archive-box']")[[1]]
@@ -42,40 +78,17 @@ listEnsemblArchives <- function(https = TRUE) {
   return(dframe)
 }
 
-listEnsembl <- function(mart = NULL, host="www.ensembl.org", version = NULL, GRCh = NULL, mirror = NULL, verbose = FALSE){
+listEnsembl <- function(mart = NULL, version = NULL, 
+                        GRCh = NULL, mirror = NULL, verbose = FALSE){
   
-  if(!is.null(mirror) & (!is.null(version) | !is.null(GRCh))){
-    warning("version or GRCh arguments can not be used together with the mirror argument.\n",  
-            "Will ignore the mirror argument and connect to default ensembl host") 
-    mirror = NULL
-  }
+  host <- .constructEnsemblURL(mirror = mirror, version = version, GRCh = GRCh)
+  port <- ifelse(grepl("https", host)[1], yes = 443, no = 80)
+  ensemblRedirect <- is.null(mirror)
   
-  if(!is.null(version)){
-    host <- paste0("e", version, ".ensembl.org")
-  }
-  if(!is.null(GRCh)){
-    if(GRCh == 37){ 
-      host = paste("https://grch",GRCh,".ensembl.org",sep="")	
-    }
-    else{
-      print("Only 37 can be specified for GRCh version")
-    }
-  }
+  httr_config <- .getEnsemblSSL()
   
-  ensemblRedirect <- TRUE
-  if(!is.null(mirror)){
-    if(!(mirror %in% c("www", "uswest", "useast", "asia"))) {
-      warning("Invalid mirror. Select a mirror from [www, uswest, useast, asia].\n",
-              "Default when no mirror is specified is to be redirected to ",
-              "www.ensembl.org")
-    } else {
-      host <- paste0(mirror, ".ensembl.org")
-      ensemblRedirect <- FALSE
-    }
-  }
-  
-  marts <- .listMarts(mart = mart, host = host, verbose = verbose, 
-                      ensemblRedirect = ensemblRedirect)
+  marts <- .listMarts(mart = mart, host = host, verbose = verbose, httr_config = httr_config,
+                      port = port, ensemblRedirect = ensemblRedirect)
   
   sel = which(marts$biomart == "ENSEMBL_MART_ENSEMBL")
   if(length(sel) > 0){ 
@@ -100,6 +113,65 @@ listEnsembl <- function(mart = NULL, host="www.ensembl.org", version = NULL, GRC
   return(marts)
 }
 
+#' creates an Ensembl URL based on the arguments provided to useEnsembl.
+#' If there are conflicting options, order of precedence is:
+#' GRCh, version, mirror
+#' Default return value is https://www.ensembl.org
+.constructEnsemblURL <- function(mirror = NULL, version = NULL, GRCh = NULL) {
+  
+  host <- NULL
+  
+  if(!is.null(mirror) & (!is.null(version) | !is.null(GRCh))){
+    warning("version or GRCh arguments cannot be used together with the mirror argument.\n", 
+            "We will ignore the mirror argument and connect to the main Ensembl site.",
+            call. = FALSE) 
+    mirror <- NULL
+  }
+  
+  if(!is.null(version) & !is.null(GRCh)) {
+    stop("version or GRCh arguments cannot be used together.\n", 
+         "Please specify only the 'version' or 'GRCh' argument.",
+         call. = FALSE) 
+  }
+  
+  if(!is.null(version)) {
+    archives <- .listEnsemblArchives(https = TRUE, httr_config = list())
+    idx <- match(version, archives[,'version'], nomatch = NA)
+    if(is.na(idx)) {
+      stop("Specified Ensembl version is not available.\n",
+           "Use listEnsemblArchives() to view available versions.",
+           call. = FALSE)
+    }
+    host <- archives[idx, 'url']
+  }	  
+  
+  if(!is.null(GRCh)){
+    if(GRCh == 37){
+      host <- paste0("https://grch", GRCh, ".ensembl.org")
+    } else {
+      warning("Only 37 can be specified for GRCh version. Using the current version.",
+              call. = FALSE)
+    }
+  }
+  
+  if(!is.null(mirror)){
+    if(!(mirror %in% c("www", "uswest", "useast", "asia"))) {
+      warning("Invalid mirror. Select a mirror from [www, uswest, useast, asia].\n",
+              "Default when no mirror is specified is to use ",
+              "www.ensembl.org which may be automatically redirected." )
+      host <- "https://www.ensembl.org"
+    } else {
+      host <- paste0("https://", mirror, ".ensembl.org")
+    }
+  }
+  
+  if(is.null(host)) {
+    host = "https://www.ensembl.org"
+  }
+  
+  return(host)
+  
+}
 
 useEnsembl <- function(biomart, dataset, host, 
                        version = NULL, GRCh = NULL, mirror = NULL, verbose = FALSE){
@@ -109,46 +181,7 @@ useEnsembl <- function(biomart, dataset, host,
          "Available Ensembl Marts can be viewed with ",
          "the function listEnsembl()")
   }
-  
-  if(!is.null(mirror) & (!is.null(version) | !is.null(GRCh))){
-    warning("version or GRCh arguments can not be used together with the mirror argument.', 
-                'We will ignore the mirror argument and connect to main Ensembl site.") 
-    mirror = NULL
-  }
-  
-  if(!is.null(version)){
-    archives <- listEnsemblArchives()
-    idx <- match(version, archives[,'version'], nomatch = NA)
-    if(is.na(idx)) {
-      stop('Specified Ensembl version is not available.\n',
-           'Use listEnsemblArchives() to view available versions.',
-           call. = FALSE)
-    }
-    host <- archives[idx, 'url']
-  }	   
-  
-  if(!is.null(GRCh)){
-    if(GRCh == 37){
-      host <- paste("https://grch",GRCh,".ensembl.org",sep="")
-    }
-    else{
-      print("Only 37 can be specified for GRCh version")
-    }
-  }
-  
-  ensemblRedirect <- TRUE
-  if(!is.null(mirror)){
-    if(!(mirror %in% c("www", "uswest", "useast", "asia"))) {
-      warning("Invalid mirror. Select a mirror from [www, uswest, useast, asia].\n",
-              "Default when no mirror is specified is to use ",
-              "www.ensembl.org which may be automatically redirected." )
-      host <- "https://www.ensembl.org"
-    } else {
-      ensemblRedirect <- FALSE
-      host <- paste0("https://", mirror, ".ensembl.org")
-    }
-  }
-  
+
   if(tolower(biomart) == "ensembl" | tolower(biomart) == "genes") {
     biomart = "ENSEMBL_MART_ENSEMBL"
   }
@@ -165,20 +198,28 @@ useEnsembl <- function(biomart, dataset, host,
     biomart = "ENSEMBL_MART_MOUSE"
   }
   
+  ## create the host URL & turn off redirection if a mirror is specified
   if(missing(host)) {
-    host = "https://www.ensembl.org"
+    host <- .constructEnsemblURL(version = version, GRCh = GRCh, mirror = mirror)
+    ensemblRedirect <- is.null(mirror)
+  } else {
+    ensemblRedirect <- FALSE
   }
   
   ## choose the port based on whether we use https or not
   port <- ifelse(grepl(pattern = "https://", x = host), 
                  yes = 443, no = 80)
   
-  ens = .useMart(biomart = biomart, 
-                 dataset = dataset, 
-                 host = host, 
-                 verbose = verbose,
-                 port = port, 
-                 ensemblRedirect = ensemblRedirect)	   
+  ## test https connection and store required settings
+  httr_config <- .getEnsemblSSL()
+  
+  ens <- .useMart(biomart = biomart, 
+                  dataset = dataset, 
+                  host = host, 
+                  verbose = verbose,
+                  port = port,
+                  ensemblRedirect = ensemblRedirect,
+                  httr_config = httr_config)	   
   return(ens)
 }
 
@@ -191,9 +232,11 @@ listEnsemblGenomes <- function(includeHosts = FALSE){
              "https://metazoa.ensembl.org/",
              "https://plants.ensembl.org/")
   
+  httr_config <- .getEnsemblSSL()
+  
   marts <- lapply(hosts, FUN = function(x) { 
     as.data.frame(
-      .listMarts(host = x, mart = NULL, 
+      .listMarts(host = x, mart = NULL, httr_config = httr_config,
                  verbose = FALSE, ensemblRedirect = FALSE, 
                  port = 443, includeHosts = includeHosts)
     ) } 
@@ -223,10 +266,15 @@ useEnsemblGenomes <- function(biomart, dataset) {
   
   host <- paste0("https://", martDetails$host)
   
+  httr_config <- .getEnsemblSSL()
+  
   ens <- .useMart(biomart = biomart, 
                  dataset = dataset, 
                  host = host, 
                  verbose = FALSE,
                  port = 443, 
-                 ensemblRedirect = FALSE)	  
+                 ensemblRedirect = FALSE,
+                 httr_config = httr_config)	  
+  
+  return(ens)
 }
\ No newline at end of file
